//===========================================================================
// Single Image Random Text Stereogram Generator
//
// Written by Andrew Wedgbury <aw@wedger.demon.co.uk>
// Copyright 2002. All Rights Reserved.
//===========================================================================

#include <iostream>
#include <time.h>

using namespace std;

#define INPUT_MAX 1024

// defaults
int d = 13;
int w = 78;
int t = 0;
int b = 0;
int l = d;
int xm = 1;
int ym = 1;
int c = 0;

char cl[][2] = { {33,47}, {64,90}, {97,122}, {33,127} };
bool rand_seeded=false;
char rand_prev=0;

//===========================================================================
char rand_char()
{
  if (!rand_seeded) {
    srand((unsigned)time(0));
    rand_seeded=true;
  }

  char rc;
  while ( (rc=(char)(cl[c][0]+((double)rand()*(cl[c][1]-cl[c][0])/RAND_MAX))) == rand_prev ) ;

  rand_prev=c;
  return rc;
}

//===========================================================================
void render_char(char* output,int x,int z)
{
  // Render a single character in 3d on the stereogram
  for (int dx=0; x+dx+z<w; dx+=d) {
    output[x+dx] = output[x+dx+z];
  }
}

//===========================================================================
int char_to_z(char c)
{
  switch (c) {

    case ' ':
      return 0;

    case 'A':
    case 'a':
    case '1':
      return 1;

    case 'B':
    case 'b':
    case '2':
      return 2;

    case 'C':
    case 'c':
    case '3':
      return 3;

    case 'D':
    case 'd':
    case '4':
      return 4;

    case 'E':
    case 'e':
    case '5':
      return 5;

  }
  return 1;
}

//===========================================================================
void render_line(const char* input)
{
  int x;
  char* output = new char[w+1];

  for (int y=0; y<ym; ++y) {

    // Create the background 'noise'
    char* noise = new char[d];
    for (x=0; x<d; ++x) {
      noise[x] = rand_char();
    }

    // Repeat across image
    for (x=0; x<w; ++x) {
      output[x] = noise[x%d];
    }

    // Render the input
    for (x=0; input[x]!=0; ++x) {
      int z=char_to_z(input[x]);
      if (z!=0) {
        int sx=l+(x*xm);
        for (int dx=0; dx<xm; ++dx) {
          render_char(output,sx+dx,z);
        }
      }
    }

    // Print the line
    output[w]='\0';
    cout << output << "\n";
  }

  delete[] output;
}

//===========================================================================
int process_options(char* argv[])
{
  for (int i=1; argv[i]!=0; ++i) {
    if (argv[i][0]=='-') {
      if (argv[i][1]=='h') {
        // Print help and quit
        cout << "Single Image Random Text Stereogram Generator\n"
             << "Copyright 2002 Andrew Wedgbury. All Rights Reserved.\n\n"
             << " Usage: banner HELLO | sirts {options}\n\n"
             << "       option default description\n"
             << "        -t n    0     Offset top\n"
             << "        -b n    0     Offset bottom\n"
             << "        -l n    13    Offset left\n"
             << "        -d n    13    Depth factor >hurts!\n"
             << "        -x n    2     Input x multiplier\n"
             << "        -y n    2     Input y multiplier\n"
             << "        -w n    78    Width of stereo image\n"
             << "        -c n    0     Chars (0=symbols, 1=A-Z, 2=a-z, 3=all)\n\n";
        return 1;

      } else if (argv[i][1]=='t' && argv[i+1]) {
        t = atoi(argv[++i]);
        if (t < 0) {
          cerr << "ERROR: -t value out of range (should be >= 0)\n";
          return 1;
        }

      } else if (argv[i][1]=='b' && argv[i+1]) {
        b = atoi(argv[++i]);
        if (b < 0) {
          cerr << "ERROR: -b value out of range (should be >= 0)\n";
          return 1;
        }

      } else if (argv[i][1]=='l' && argv[i+1]) {
        l = atoi(argv[++i]);

      } else if (argv[i][1]=='d' && argv[i+1]) {
        d = atoi(argv[++i]);
        if (d <= 2) {
          cerr << "ERROR: -d value out of range (should be > 2)\n";
          return 1;
        }

      } else if (argv[i][1]=='x' && argv[i+1]) {
        xm = atoi(argv[++i]);
        if (xm <= 0) {
          cerr << "ERROR: -x value out of range (should be > 0)\n";
          return 1;
        }

      } else if (argv[i][1]=='y' && argv[i+1]) {
        ym = atoi(argv[++i]);
        if (ym <= 0) {
          cerr << "ERROR: -y value out of range (should be > 0)\n";
          return 1;
        }

      } else if (argv[i][1]=='w' && argv[i+1]) {
        w = atoi(argv[++i]);
        if (w <= 0) {
          cerr << "ERROR: -w value out of range (should be > 0)\n";
          return 1;
        }

      } else if (argv[i][1]=='c' && argv[i+1]) {
        c = atoi(argv[++i]);
        int maxc = sizeof(cl)/(sizeof(cl[0]))-1;
        if (c < 0 || c > maxc) {
          cerr << "ERROR: -c value out of range (should be 0-" << maxc << ")\n";
          return 1;
        }

      }
    }
  }
  return 0;
}

//===========================================================================
int main(int argc,char* argv[],char* envp[])
{
  // Process command line options
  if (process_options(argv)) {
    return 1;
  }

  // Create buffer
  char* input = new char[INPUT_MAX];

  // Add top spacing
  int i;
  for (i=0; i<t; ++i) render_line("");

  // Loop, reading and rendering each line
  while (!cin.eof()) {
    cin.getline(input,1024);
    render_line(input);
  }

  // Add bottom spacing
  for (i=0; i<b; ++i) render_line("");

  // Clean up
  delete[] input;
  return 0;
}

